package lcsb.mapviewer.api.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.model.user.BasicPrivilege;
import lcsb.mapviewer.model.user.ObjectPrivilege;
import lcsb.mapviewer.model.user.PrivilegeType;
import lcsb.mapviewer.model.user.User;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.interfaces.ILayoutService;
import lcsb.mapviewer.services.interfaces.IUserService;

@Transactional(value = "txManager")
public class UserRestImpl {

	@Autowired
	private IUserService	 userService;

	@Autowired
	private ILayoutService layoutService;

	/**
	 * @return the userService
	 * @see #userService
	 */
	public IUserService getUserService() {
		return userService;
	}

	/**
	 * @param userService
	 *          the userService to set
	 * @see #userService
	 */
	public void setUserService(IUserService userService) {
		this.userService = userService;
	}

	public Map<String, Object> getUser(String token, String userId, String columns) throws SecurityException {
		User ownUserData = userService.getUserByToken(token);
		Integer id = null;
		if (userId != null && !userId.isEmpty()) {
			id = Integer.valueOf(userId);
		} else {
			id = ownUserData.getId();
		}

		Set<String> columnSet = createUserColumnSet(columns);

		boolean isAdmin = userService.userHasPrivilege(ownUserData, PrivilegeType.USER_MANAGEMENT);

		if (ownUserData.getId().equals(id)) {
			return prepareUse(ownUserData, columnSet, true);
		} else if (isAdmin) {
			User user = userService.getUserById(id);
			if (user == null) {
				throw new SecurityException("You cannot access data of the user with given id");
			}
			return prepareUse(user, columnSet, isAdmin);
		} else {
			throw new SecurityException("You cannot access data of the user with given id");
		}
	}

	private Set<String> createUserColumnSet(String columns) {
		Set<String> columnsSet = new HashSet<>();
		if (columns.equals("")) {
			columnsSet.add("id");
			columnsSet.add("login");
			columnsSet.add("name");
			columnsSet.add("surname");
			columnsSet.add("email");
			columnsSet.add("minColor");
			columnsSet.add("maxColor");
			columnsSet.add("removed");
			columnsSet.add("privileges");
		} else {
			for (String str : columns.split(",")) {
				columnsSet.add(str);
			}
		}
		return columnsSet;
	}

	private Map<String, Object> prepareUse(User user, Set<String> columnsSet, boolean admin) {
		Map<String, Object> result = new HashMap<>();
		for (String string : columnsSet) {
			String column = string.toLowerCase();
			Object value = null;
			if (column.equals("id") || column.equals("idobject")) {
				value = user.getId();
			} else if (column.equals("name")) {
				value = user.getName();
			} else if (column.equals("surname")) {
				value = user.getSurname();
			} else if (column.equals("login")) {
				value = user.getLogin();
			} else if (column.equals("email")) {
				value = user.getEmail();
			} else if (column.equals("mincolor")) {
				value = user.getMinColor();
			} else if (column.equals("maxcolor")) {
				value = user.getMaxColor();
			} else if (column.equals("removed")) {
				value = user.isRemoved();
			} else if (column.equals("privileges") && admin) {
				value = preparePrivileges(user);
			} else {
				value = "Unknown column";
			}
			result.put(string, value);
		}
		return result;
	}

	private List<Map<String, Object>> preparePrivileges(User user) {
		List<Map<String, Object>> result = new ArrayList<>();
		for (BasicPrivilege privilege : user.getPrivileges()) {
			if (privilege instanceof ObjectPrivilege) {
				result.add(prepareObjectPrivilege((ObjectPrivilege) privilege));
			} else {
				result.add(prepareBasicPrivilege(privilege));
			}
		}
		Map<String, Object> customLayouts = new HashMap<>();
		customLayouts.put("type", "CUSTOM_LAYOUTS_AVAILABLE");
		customLayouts.put("value", layoutService.getAvailableCustomLayoutsNumber(user));
		result.add(customLayouts);
		return result;
	}

	private Map<String, Object> prepareObjectPrivilege(ObjectPrivilege privilege) {
		Map<String, Object> result = new HashMap<>();
		result.put("type", privilege.getType());
		result.put("value", privilege.getLevel());
		result.put("objectId", privilege.getIdObject());
		return result;
	}

	private Map<String, Object> prepareBasicPrivilege(BasicPrivilege privilege) {
		Map<String, Object> result = new HashMap<>();
		if (privilege.getClass().equals(BasicPrivilege.class)) {
			result.put("type", privilege.getType());
			result.put("value", privilege.getLevel());
			return result;
		} else {
			throw new InvalidArgumentException("Don't know how to handle class: " + privilege.getClass());
		}
	}

	/**
	 * @return the layoutService
	 * @see #layoutService
	 */
	public ILayoutService getLayoutService() {
		return layoutService;
	}

	/**
	 * @param layoutService
	 *          the layoutService to set
	 * @see #layoutService
	 */
	public void setLayoutService(ILayoutService layoutService) {
		this.layoutService = layoutService;
	}

}
